"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useScrollSpy = void 0;
const react_1 = require("react");
const throttle_1 = require("../utils/throttle");
/**
 * @param ScrollSpyParams Optional params
 * @param ScrollSpyParams.offsetPx Distance from Y coordinate of the base (px)
 * @param ScrollSpyParams.throttleMs Interval of update processing (ms)
 * @returns [ activeElement, actions ] - Two return values are returned as an array
 * @returns activeElement - DOM element activated based on scroll position (only one)
 * @returns actions - Actions are used to control registered DOM elements
 */
const useScrollSpy = ({ offsetPx = 0, throttleMs = 50, } = {}) => {
    const [activeElement, setActiveElement] = (0, react_1.useState)(null);
    const elements = (0, react_1.useRef)({});
    const registerElement = (0, react_1.useCallback)(({ key, element, }) => {
        const prevElements = elements.current;
        elements.current = {
            ...prevElements,
            [key]: element,
        };
    }, []);
    const unregisterElement = (0, react_1.useCallback)(({ key }) => {
        const deletedElements = { ...elements.current };
        delete deletedElements[key];
        elements.current = deletedElements;
    }, []);
    const resetElements = (0, react_1.useCallback)(() => {
        elements.current = {};
    }, []);
    const updateActiveElement = (0, react_1.useCallback)(() => {
        const currentElements = elements.current;
        const elementKeys = Object.keys(currentElements);
        const { key: activeKey } = elementKeys.reduce((prevActive, key) => {
            const elementElement = currentElements[key].current;
            if (!elementElement) {
                return prevActive;
            }
            const elementPositionTop = elementElement.getBoundingClientRect().top;
            const isPassed = elementPositionTop + offsetPx < 0;
            if (!isPassed) {
                return prevActive;
            }
            if (!prevActive.key) {
                return {
                    key,
                    elementPositionTop,
                };
            }
            const { elementPositionTop: prevElementPositionTop = 0 } = prevActive;
            const isHigherThanPrev = elementPositionTop >= prevElementPositionTop;
            if (isHigherThanPrev) {
                return {
                    key,
                    elementPositionTop,
                };
            }
            else {
                return prevActive;
            }
        }, {});
        if (!activeKey) {
            setActiveElement(null);
            return;
        }
        setActiveElement({
            key: activeKey,
            value: currentElements[activeKey],
        });
    }, [offsetPx]);
    (0, react_1.useEffect)(() => {
        const InvokeUpdate = (0, throttle_1.throttle)(updateActiveElement, throttleMs);
        window.addEventListener("scroll", InvokeUpdate);
        return () => {
            window.removeEventListener("scroll", InvokeUpdate);
        };
    }, [throttleMs, updateActiveElement]);
    const actions = {
        registerElement,
        unregisterElement,
        resetElements,
    };
    return [activeElement, actions];
};
exports.useScrollSpy = useScrollSpy;
//# sourceMappingURL=useScrollSpy.js.map